@@ -1,30 +1,47 @@
 # interfaces.py
-# Copyright (C) 2005, 2006, 2007, 2008 Michael Bayer mike_mp@zzzcomputing.com
+# Copyright (C) 2005, 2006, 2007, 2008, 2009 Michael Bayer mike_mp@zzzcomputing.com
 #
 # This module is part of SQLAlchemy and is released under
 # the MIT License: http://www.opensource.org/licenses/mit-license.php
 
-"""Semi-private implementation objects which form the basis
-of ORM-mapped attributes, query options and mapper extension.
+"""
+
+Semi-private module containing various base classes used throughout the ORM.
+
+Defines the extension classes :class:`MapperExtension`, 
+:class:`SessionExtension`, and :class:`AttributeExtension` as
+well as other user-subclassable extension objects.
 
-Defines the [sqlalchemy.orm.interfaces#MapperExtension] class,
-which can be end-user subclassed to add event-based functionality
-to mappers.  The remainder of this module is generally private to the
-ORM.
 """
 
 from itertools import chain
-from sqlalchemy import exceptions, logging, util
+
+import sqlalchemy.exceptions as sa_exc
+from sqlalchemy import log, util
 from sqlalchemy.sql import expression
+
 class_mapper = None
+collections = None
 
-__all__ = ['EXT_CONTINUE', 'EXT_STOP', 'EXT_PASS', 'MapperExtension',
-           'MapperProperty', 'PropComparator', 'StrategizedProperty',
-           'build_path', 'MapperOption',
-           'ExtensionOption', 'PropertyOption',
-           'AttributeExtension', 'StrategizedOption', 'LoaderStrategy' ]
+__all__ = (
+    'AttributeExtension',
+    'EXT_CONTINUE',
+    'EXT_STOP',
+    'ExtensionOption',
+    'InstrumentationManager',
+    'LoaderStrategy',
+    'MapperExtension',
+    'MapperOption',
+    'MapperProperty',
+    'PropComparator',
+    'PropertyOption',
+    'SessionExtension',
+    'StrategizedOption',
+    'StrategizedProperty',
+    'build_path',
+    )
 
-EXT_CONTINUE = EXT_PASS = util.symbol('EXT_CONTINUE')
+EXT_CONTINUE = util.symbol('EXT_CONTINUE')
 EXT_STOP = util.symbol('EXT_STOP')
 
 ONETOMANY = util.symbol('ONETOMANY')
@@ -44,10 +61,7 @@ class MapperExtension(object):
     these exception cases, any return value other than EXT_CONTINUE or
     EXT_STOP will be interpreted as equivalent to EXT_STOP.
 
-    EXT_PASS is a synonym for EXT_CONTINUE and is provided for backward
-    compatibility.
     """
-
     def instrument_class(self, mapper, class_):
         return EXT_CONTINUE
 
@@ -57,79 +71,15 @@ class MapperExtension(object):
     def init_failed(self, mapper, class_, oldinit, instance, args, kwargs):
         return EXT_CONTINUE
 
-    def get_session(self):
-        """Retrieve a contextual Session instance with which to
-        register a new object.
-
-        Note: this is not called if a session is provided with the
-        `__init__` params (i.e. `_sa_session`).
-        """
-
-        return EXT_CONTINUE
-
-    def load(self, query, *args, **kwargs):
-        """Override the `load` method of the Query object.
-
-        The return value of this method is used as the result of
-        ``query.load()`` if the value is anything other than EXT_CONTINUE.
-        """
-
-        return EXT_CONTINUE
-
-    def get(self, query, *args, **kwargs):
-        """Override the `get` method of the Query object.
-
-        The return value of this method is used as the result of
-        ``query.get()`` if the value is anything other than EXT_CONTINUE.
-        """
-
-        return EXT_CONTINUE
-
-    def get_by(self, query, *args, **kwargs):
-        """Override the `get_by` method of the Query object.
-
-        The return value of this method is used as the result of
-        ``query.get_by()`` if the value is anything other than
-        EXT_CONTINUE.
-
-        DEPRECATED.
-        """
-
-        return EXT_CONTINUE
-
-    def select_by(self, query, *args, **kwargs):
-        """Override the `select_by` method of the Query object.
-
-        The return value of this method is used as the result of
-        ``query.select_by()`` if the value is anything other than
-        EXT_CONTINUE.
-
-        DEPRECATED.
-        """
-
-        return EXT_CONTINUE
-
-    def select(self, query, *args, **kwargs):
-        """Override the `select` method of the Query object.
-
-        The return value of this method is used as the result of
-        ``query.select()`` if the value is anything other than
-        EXT_CONTINUE.
-
-        DEPRECATED.
-        """
-
-        return EXT_CONTINUE
-
-
     def translate_row(self, mapper, context, row):
         """Perform pre-processing on the given result row and return a
         new row instance.
 
-        This is called as the very first step in the ``_instance()``
-        method.
-        """
+        This is called when the mapper first receives a row, before
+        the object identity or the instance itself has been derived
+        from that row.
 
+        """
         return EXT_CONTINUE
 
     def create_instance(self, mapper, selectcontext, row, class_):
@@ -153,8 +103,8 @@ class MapperExtension(object):
 
         return value
           A new object instance, or EXT_CONTINUE
-        """
 
+        """
         return EXT_CONTINUE
 
     def append_result(self, mapper, selectcontext, row, instance, result, **flags):
@@ -184,25 +134,49 @@ class MapperExtension(object):
 
         \**flags
           extra information about the row, same as criterion in
-          `create_row_processor()` method of [sqlalchemy.orm.interfaces#MapperProperty]
+          ``create_row_processor()`` method of :class:`~sqlalchemy.orm.interfaces.MapperProperty`
         """
 
         return EXT_CONTINUE
 
     def populate_instance(self, mapper, selectcontext, row, instance, **flags):
-        """Receive a newly-created instance before that instance has
+        """Receive an instance before that instance has
         its attributes populated.
 
-        The normal population of attributes is according to each
-        attribute's corresponding MapperProperty (which includes
-        column-based attributes as well as relationships to other
-        classes).  If this method returns EXT_CONTINUE, instance
-        population will proceed normally.  If any other value or None
-        is returned, instance population will not proceed, giving this
-        extension an opportunity to populate the instance itself, if
-        desired.
-        """
+        This usually corresponds to a newly loaded instance but may
+        also correspond to an already-loaded instance which has
+        unloaded attributes to be populated.  The method may be called
+        many times for a single instance, as multiple result rows are
+        used to populate eagerly loaded collections.
 
+        If this method returns EXT_CONTINUE, instance population will
+        proceed normally.  If any other value or None is returned,
+        instance population will not proceed, giving this extension an
+        opportunity to populate the instance itself, if desired.
+
+        As of 0.5, most usages of this hook are obsolete.  For a
+        generic "object has been newly created from a row" hook, use
+        ``reconstruct_instance()``, or the ``@orm.reconstructor``
+        decorator.
+
+        """
+        return EXT_CONTINUE
+
+    def reconstruct_instance(self, mapper, instance):
+        """Receive an object instance after it has been created via
+        ``__new__``, and after initial attribute population has
+        occurred.
+
+        This typicically occurs when the instance is created based on
+        incoming result rows, and is only called once for that
+        instance's lifetime.
+
+        Note that during a result-row load, this method is called upon
+        the first row received for this instance. If eager loaders are
+        set to further populate collections on the instance, those
+        will *not* yet be completely loaded.
+
+        """
         return EXT_CONTINUE
 
     def before_insert(self, mapper, connection, instance):
@@ -212,11 +186,12 @@ class MapperExtension(object):
         This is a good place to set up primary key values and such
         that aren't handled otherwise.
 
-        Column-based attributes can be modified within this method which will
-        result in the new value being inserted.  However *no* changes to the overall
-        flush plan can be made; this means any collection modification or
-        save() operations which occur within this method will not take effect
-        until the next flush call.
+        Column-based attributes can be modified within this method
+        which will result in the new value being inserted.  However
+        *no* changes to the overall flush plan can be made; this means
+        any collection modification or save() operations which occur
+        within this method will not take effect until the next flush
+        call.
 
         """
 
@@ -276,6 +251,81 @@ class MapperExtension(object):
 
         return EXT_CONTINUE
 
+class SessionExtension(object):
+    """An extension hook object for Sessions.  Subclasses may be installed into a Session
+    (or sessionmaker) using the ``extension`` keyword argument.
+    """
+
+    def before_commit(self, session):
+        """Execute right before commit is called.
+
+        Note that this may not be per-flush if a longer running transaction is ongoing."""
+
+    def after_commit(self, session):
+        """Execute after a commit has occured.
+
+        Note that this may not be per-flush if a longer running transaction is ongoing."""
+
+    def after_rollback(self, session):
+        """Execute after a rollback has occured.
+
+        Note that this may not be per-flush if a longer running transaction is ongoing."""
+
+    def before_flush(self, session, flush_context, instances):
+        """Execute before flush process has started.
+
+        `instances` is an optional list of objects which were passed to the ``flush()``
+        method.
+        """
+
+    def after_flush(self, session, flush_context):
+        """Execute after flush has completed, but before commit has been called.
+
+        Note that the session's state is still in pre-flush, i.e. 'new', 'dirty',
+        and 'deleted' lists still show pre-flush state as well as the history
+        settings on instance attributes."""
+
+    def after_flush_postexec(self, session, flush_context):
+        """Execute after flush has completed, and after the post-exec state occurs.
+
+        This will be when the 'new', 'dirty', and 'deleted' lists are in their final
+        state.  An actual commit() may or may not have occured, depending on whether or not
+        the flush started its own transaction or participated in a larger transaction.
+        """
+
+    def after_begin(self, session, transaction, connection):
+        """Execute after a transaction is begun on a connection
+
+        `transaction` is the SessionTransaction. This method is called after an
+        engine level transaction is begun on a connection.
+        """
+
+    def after_attach(self, session, instance):
+        """Execute after an instance is attached to a session.
+
+        This is called after an add, delete or merge.
+        """
+
+    def after_bulk_update(self, session, query, query_context, result):
+        """Execute after a bulk update operation to the session.
+
+        This is called after a session.query(...).update()
+
+        `query` is the query object that this update operation was called on.
+        `query_context` was the query context object.
+        `result` is the result object returned from the bulk operation.
+        """
+
+    def after_bulk_delete(self, session, query, query_context, result):
+        """Execute after a bulk delete operation to the session.
+
+        This is called after a session.query(...).delete()
+
+        `query` is the query object that this delete operation was called on.
+        `query_context` was the query context object.
+        `result` is the result object returned from the bulk operation.
+        """
+    
 class MapperProperty(object):
     """Manage the relationship of a ``Mapper`` to a single class
     attribute, as well as that attribute as it appears on individual
@@ -283,7 +333,7 @@ class MapperProperty(object):
     attribute access, loading behavior, and dependency calculations.
     """
 
-    def setup(self, querycontext, **kwargs):
+    def setup(self, context, entity, path, adapter, **kwargs):
         """Called by Query for the purposes of constructing a SQL statement.
 
         Each MapperProperty associated with the target mapper processes the
@@ -293,8 +343,8 @@ class MapperProperty(object):
 
         pass
 
-    def create_row_processor(self, selectcontext, mapper, row):
-        """Return a 3-tuple consiting of two row processing functions and an instance post-processing function.
+    def create_row_processor(self, selectcontext, path, mapper, row, adapter):
+        """Return a 2-tuple consiting of two row processing functions and an instance post-processing function.
 
         Input arguments are the query.SelectionContext and the *first*
         applicable row of a result set obtained within
@@ -305,32 +355,24 @@ class MapperProperty(object):
         columns present in the row (which will be the same columns present in
         all rows) are used to determine the presence and behavior of the
         returned callables.  The callables will then be used to process all
-        rows and to post-process all instances, respectively.
+        rows and instances.
 
         Callables are of the following form::
 
-            def new_execute(instance, row, **flags):
-                # process incoming instance and given row.  the instance is
+            def new_execute(state, row, **flags):
+                # process incoming instance state and given row.  the instance is
                 # "new" and was just created upon receipt of this row.
                 # flags is a dictionary containing at least the following
                 # attributes:
                 #   isnew - indicates if the instance was newly created as a
                 #           result of reading this row
                 #   instancekey - identity key of the instance
-                # optional attribute:
-                #   ispostselect - indicates if this row resulted from a
-                #                  'post' select of additional tables/columns
 
-            def existing_execute(instance, row, **flags):
-                # process incoming instance and given row.  the instance is
+            def existing_execute(state, row, **flags):
+                # process incoming instance state and given row.  the instance is
                 # "existing" and was created based on a previous row.
 
-            def post_execute(instance, **flags):
-                # process instance after all result rows have been processed.
-                # this function should be used to issue additional selections
-                # in order to eagerly load additional properties.
-
-            return (new_execute, existing_execute, post_execute)
+            return (new_execute, existing_execute)
 
         Either of the three tuples can be ``None`` in which case no function
         is called.
@@ -345,32 +387,21 @@ class MapperProperty(object):
         See PropertyLoader for the related instance implementation.
         """
 
-        return iter([])
-
-    def get_criterion(self, query, key, value):
-        """Return a ``WHERE`` clause suitable for this
-        ``MapperProperty`` corresponding to the given key/value pair,
-        where the key is a column or object property name, and value
-        is a value to be matched.  This is only picked up by
-        ``PropertyLoaders``.
-
-        This is called by a ``Query``'s ``join_by`` method to formulate a set
-        of key/value pairs into a ``WHERE`` criterion that spans multiple
-        tables if needed.
-        """
-
-        return None
+        return iter(())
 
     def set_parent(self, parent):
         self.parent = parent
 
-    def init(self, key, parent):
+    def instrument_class(self, mapper):
+        raise NotImplementedError()
+        
+    def init(self):
         """Called after all mappers are compiled to assemble
         relationships between mappers, establish instrumented class
         attributes.
         """
 
-        self.key = key
+        self._compiled = True
         self.do_init()
 
     def do_init(self):
@@ -402,7 +433,7 @@ class MapperProperty(object):
 
         return not self.parent.non_primary
 
-    def merge(self, session, source, dest):
+    def merge(self, session, source, dest, dont_load, _recursive):
         """Merge the attribute represented by this ``MapperProperty``
         from source to destination object"""
 
@@ -422,56 +453,57 @@ class MapperProperty(object):
 
 class PropComparator(expression.ColumnOperators):
     """defines comparison operations for MapperProperty objects.
-    
+
     PropComparator instances should also define an accessor 'property'
     which returns the MapperProperty associated with this
     PropComparator.
     """
 
-    def expression_element(self):
-        return self.clause_element()
+    def __init__(self, prop, mapper, adapter=None):
+        self.prop = self.property = prop
+        self.mapper = mapper
+        self.adapter = adapter
 
-    def contains_op(a, b):
-        return a.contains(b)
-    contains_op = staticmethod(contains_op)
+    def __clause_element__(self):
+        raise NotImplementedError("%r" % self)
 
+    def adapted(self, adapter):
+        """Return a copy of this PropComparator which will use the given adaption function
+        on the local side of generated expressions.
+        
+        """
+        return self.__class__(self.prop, self.mapper, adapter)
+        
+    @staticmethod
     def any_op(a, b, **kwargs):
         return a.any(b, **kwargs)
-    any_op = staticmethod(any_op)
 
+    @staticmethod
     def has_op(a, b, **kwargs):
         return a.has(b, **kwargs)
-    has_op = staticmethod(has_op)
 
-    def __init__(self, prop):
-        self.prop = self.property = prop
-    
+    @staticmethod
     def of_type_op(a, class_):
         return a.of_type(class_)
-    of_type_op = staticmethod(of_type_op)
-    
+
     def of_type(self, class_):
         """Redefine this object in terms of a polymorphic subclass.
-        
+
         Returns a new PropComparator from which further criterion can be evaluated.
 
         e.g.::
-        
+
             query.join(Company.employees.of_type(Engineer)).\\
                filter(Engineer.name=='foo')
-              
+
         \class_
             a class or mapper indicating that criterion will be against
             this specific subclass.
 
-         
+
         """
-        
+
         return self.operate(PropComparator.of_type_op, class_)
-        
-    def contains(self, other):
-        """Return true if this collection contains other"""
-        return self.operate(PropComparator.contains_op, other)
 
     def any(self, criterion=None, **kwargs):
         """Return true if this collection contains any member that meets the given criterion.
@@ -511,44 +543,51 @@ class StrategizedProperty(MapperProperty):
     ``StrategizedOption`` objects via the Query.options() method.
     """
 
-    def _get_context_strategy(self, context):
-        path = context.path
-        return self._get_strategy(context.attributes.get(("loaderstrategy", path), self.strategy.__class__))
+    def __get_context_strategy(self, context, path):
+        cls = context.attributes.get(("loaderstrategy", path), None)
+        if cls:
+            try:
+                return self.__all_strategies[cls]
+            except KeyError:
+                return self.__init_strategy(cls)
+        else:
+            return self.strategy
 
     def _get_strategy(self, cls):
         try:
-            return self._all_strategies[cls]
+            return self.__all_strategies[cls]
         except KeyError:
-            # cache the located strategy per class for faster re-lookup
-            strategy = cls(self)
-            strategy.init()
-            self._all_strategies[cls] = strategy
-            return strategy
+            return self.__init_strategy(cls)
 
-    def setup(self, querycontext, **kwargs):
-        self._get_context_strategy(querycontext).setup_query(querycontext, **kwargs)
+    def __init_strategy(self, cls):
+        self.__all_strategies[cls] = strategy = cls(self)
+        strategy.init()
+        return strategy
 
-    def create_row_processor(self, selectcontext, mapper, row):
-        return self._get_context_strategy(selectcontext).create_row_processor(selectcontext, mapper, row)
+    def setup(self, context, entity, path, adapter, **kwargs):
+        self.__get_context_strategy(context, path + (self.key,)).setup_query(context, entity, path, adapter, **kwargs)
+
+    def create_row_processor(self, context, path, mapper, row, adapter):
+        return self.__get_context_strategy(context, path + (self.key,)).create_row_processor(context, path, mapper, row, adapter)
 
     def do_init(self):
-        self._all_strategies = {}
-        self.strategy = self._get_strategy(self.strategy_class)
+        self.__all_strategies = {}
+        self.strategy = self.__init_strategy(self.strategy_class)
         if self.is_primary():
             self.strategy.init_class_attribute()
 
-def build_path(mapper, key, prev=None):
+def build_path(entity, key, prev=None):
     if prev:
-        return prev + (mapper.base_mapper, key)
+        return prev + (entity, key)
     else:
-        return (mapper.base_mapper, key)
+        return (entity, key)
 
 def serialize_path(path):
     if path is None:
         return None
 
     return [
-        (mapper.class_, mapper.entity_name, key)
+        (mapper.class_, key)
         for mapper, key in [(path[i], path[i+1]) for i in range(0, len(path)-1, 2)]
     ]
 
@@ -561,7 +600,7 @@ def deserialize_path(path):
         from sqlalchemy.orm import class_mapper
 
     return tuple(
-        chain(*[(class_mapper(cls, entity), key) for cls, entity, key in path])
+        chain(*[(class_mapper(cls), key) for cls, key in path])
     )
 
 class MapperOption(object):
@@ -585,9 +624,9 @@ class ExtensionOption(MapperOption):
         self.ext = ext
 
     def process_query(self, query):
-        query._extension = query._extension.copy()
-        query._extension.insert(self.ext)
-
+        entity = query._generate_mapper_zero()
+        entity.extension = entity.extension.copy()
+        entity.extension.push(self.ext)
 
 class PropertyOption(MapperOption):
     """A MapperOption that is applied to a property off the mapper or
@@ -605,76 +644,117 @@ class PropertyOption(MapperOption):
         self._process(query, False)
 
     def _process(self, query, raiseerr):
-        if self._should_log_debug:
-            self.logger.debug("applying option to Query, property key '%s'" % self.key)
-        paths = self._get_paths(query, raiseerr)
+        paths = self.__get_paths(query, raiseerr)
         if paths:
             self.process_query_property(query, paths)
 
     def process_query_property(self, query, paths):
         pass
 
-    def _get_paths(self, query, raiseerr):
+    def __find_entity(self, query, mapper, raiseerr):
+        from sqlalchemy.orm.util import _class_to_mapper, _is_aliased_class
+
+        if _is_aliased_class(mapper):
+            searchfor = mapper
+        else:
+            searchfor = _class_to_mapper(mapper).base_mapper
+
+        for ent in query._mapper_entities:
+            if ent.path_entity is searchfor:
+                return ent
+        else:
+            if raiseerr:
+                raise sa_exc.ArgumentError("Can't find entity %s in Query.  Current list: %r" % (searchfor, [str(m.path_entity) for m in query._entities]))
+            else:
+                return None
+
+    def __get_paths(self, query, raiseerr):
         path = None
+        entity = None
         l = []
+
         current_path = list(query._current_path)
 
         if self.mapper:
-            global class_mapper
-            if class_mapper is None:
-                from sqlalchemy.orm import class_mapper
-            mapper = self.mapper
-            if isinstance(self.mapper, type):
-                mapper = class_mapper(mapper)
-            if mapper is not query.mapper and mapper not in [q.mapper for q in query._entities]:
-                raise exceptions.ArgumentError("Can't find entity %s in Query.  Current list: %r" % (str(mapper), [str(m) for m in query._entities]))
-        else:
-            mapper = query.mapper
-        if isinstance(self.key, basestring):
-            tokens = self.key.split('.')
-        else:
-            tokens = util.to_list(self.key)
-            
-        for token in tokens:
-            if isinstance(token, basestring):
-                prop = mapper.get_property(token, resolve_synonyms=True, raiseerr=raiseerr)
-            elif isinstance(token, PropComparator):
-                prop = token.property
-                token = prop.key
-                    
+            entity = self.__find_entity(query, self.mapper, raiseerr)
+            mapper = entity.mapper
+            path_element = entity.path_entity
+
+        for key in util.to_list(self.key):
+            if isinstance(key, basestring):
+                tokens = key.split('.')
             else:
-                raise exceptions.ArgumentError("mapper option expects string key or list of attributes")
-                
-            if current_path and token == current_path[1]:
-                current_path = current_path[2:]
-                continue
-                
-            if prop is None:
-                return []
-            path = build_path(mapper, prop.key, path)
-            l.append(path)
-            if getattr(token, '_of_type', None):
-                mapper = token._of_type
-            else:
-                mapper = getattr(prop, 'mapper', None)
+                tokens = [key]
+            for token in tokens:
+                if isinstance(token, basestring):
+                    if not entity:
+                        entity = query._entity_zero()
+                        path_element = entity.path_entity
+                        mapper = entity.mapper
+                    prop = mapper.get_property(token, resolve_synonyms=True, raiseerr=raiseerr)
+                    key = token
+                elif isinstance(token, PropComparator):
+                    prop = token.property
+                    if not entity:
+                        entity = self.__find_entity(query, token.parententity, raiseerr)
+                        if not entity:
+                            return []
+                        path_element = entity.path_entity
+                    key = prop.key
+                else:
+                    raise sa_exc.ArgumentError("mapper option expects string key or list of attributes")
+
+                if current_path and key == current_path[1]:
+                    current_path = current_path[2:]
+                    continue
+
+                if prop is None:
+                    return []
+
+                path = build_path(path_element, prop.key, path)
+                l.append(path)
+                if getattr(token, '_of_type', None):
+                    path_element = mapper = token._of_type
+                else:
+                    path_element = mapper = getattr(prop, 'mapper', None)
+                if path_element:
+                    path_element = path_element.base_mapper
+
         return l
 
-PropertyOption.logger = logging.class_logger(PropertyOption)
-PropertyOption._should_log_debug = logging.is_debug_enabled(PropertyOption.logger)
-
 class AttributeExtension(object):
-    """An abstract class which specifies `append`, `delete`, and `set`
-    event handlers to be attached to an object property.
+    """An event handler for individual attribute change events.
+    
+    AttributeExtension is assembled within the descriptors associated 
+    with a mapped class. 
+    
     """
 
-    def append(self, obj, child, initiator):
+    def append(self, state, value, initiator):
+        """Receive a collection append event.
+        
+        The returned value will be used as the actual value to be
+        appended.
+        
+        """
+        return value
+
+    def remove(self, state, value, initiator):
+        """Receive a remove event.
+        
+        No return value is defined.
+        
+        """
         pass
 
-    def remove(self, obj, child, initiator):
-        pass
-
-    def set(self, obj, child, oldchild, initiator):
-        pass
+    def set(self, state, value, oldvalue, initiator):
+        """Receive a set event.
+        
+        The returned value will be used as the actual value to be
+        set.
+        
+        """
+        return value
 
 
 class StrategizedOption(PropertyOption):
@@ -724,18 +804,19 @@ class LoaderStrategy(object):
     def __init__(self, parent):
         self.parent_property = parent
         self.is_class_level = False
-
-    def init(self):
         self.parent = self.parent_property.parent
         self.key = self.parent_property.key
 
+    def init(self):
+        raise NotImplementedError("LoaderStrategy")
+
     def init_class_attribute(self):
         pass
 
-    def setup_query(self, context, **kwargs):
+    def setup_query(self, context, entity, path, adapter, **kwargs):
         pass
 
-    def create_row_processor(self, selectcontext, mapper, row):
+    def create_row_processor(self, selectcontext, path, mapper, row, adapter):
         """Return row processing functions which fulfill the contract specified
         by MapperProperty.create_row_processor.
 
@@ -744,3 +825,74 @@ class LoaderStrategy(object):
         """
 
         raise NotImplementedError()
+
+    def __str__(self):
+        return str(self.parent_property)
+
+    def debug_callable(self, fn, logger, announcement, logfn):
+        if announcement:
+            logger.debug(announcement)
+        if logfn:
+            def call(*args, **kwargs):
+                logger.debug(logfn(*args, **kwargs))
+                return fn(*args, **kwargs)
+            return call
+        else:
+            return fn
+
+class InstrumentationManager(object):
+    """User-defined class instrumentation extension."""
+
+    # r4361 added a mandatory (cls) constructor to this interface.
+    # given that, perhaps class_ should be dropped from all of these
+    # signatures.
+
+    def __init__(self, class_):
+        pass
+
+    def manage(self, class_, manager):
+        setattr(class_, '_default_class_manager', manager)
+
+    def dispose(self, class_, manager):
+        delattr(class_, '_default_class_manager')
+
+    def manager_getter(self, class_):
+        def get(cls):
+            return cls._default_class_manager
+        return get
+
+    def instrument_attribute(self, class_, key, inst):
+        pass
+
+    def install_descriptor(self, class_, key, inst):
+        setattr(class_, key, inst)
+
+    def uninstall_descriptor(self, class_, key):
+        delattr(class_, key)
+
+    def install_member(self, class_, key, implementation):
+        setattr(class_, key, implementation)
+
+    def uninstall_member(self, class_, key):
+        delattr(class_, key)
+
+    def instrument_collection_class(self, class_, key, collection_class):
+        global collections
+        if collections is None:
+            from sqlalchemy.orm import collections
+        return collections.prepare_instrumentation(collection_class)
+
+    def get_instance_dict(self, class_, instance):
+        return instance.__dict__
+
+    def initialize_instance_dict(self, class_, instance):
+        pass
+
+    def install_state(self, class_, instance, state):
+        setattr(instance, '_default_state', state)
+
+    def remove_state(self, class_, instance):
+        delattr(instance, '_default_state', state)
+
+    def state_getter(self, class_):
+        return lambda instance: getattr(instance, '_default_state')
