@@ -1,5 +1,5 @@
 # properties.py
-# Copyright (C) 2005, 2006, 2007, 2008 Michael Bayer mike_mp@zzzcomputing.com
+# Copyright (C) 2005, 2006, 2007, 2008, 2009 Michael Bayer mike_mp@zzzcomputing.com
 #
 # This module is part of SQLAlchemy and is released under
 # the MIT License: http://www.opensource.org/licenses/mit-license.php
@@ -32,24 +32,56 @@ class ColumnProperty(StrategizedProperty):
     """Describes an object attribute that corresponds to a table column."""
 
     def __init__(self, *columns, **kwargs):
-        """The list of `columns` describes a single object
-        property. If there are multiple tables joined together for the
-        mapper, this list represents the equivalent column as it
-        appears across each table.
-        """
+        """Construct a ColumnProperty.
 
+        :param \*columns: The list of `columns` describes a single
+          object property. If there are multiple tables joined
+          together for the mapper, this list represents the equivalent
+          column as it appears across each table.
+
+        :param group:
+
+        :param deferred:
+
+        :param comparator_factory:
+
+        :param descriptor:
+
+        :param extension:
+
+        """
         self.columns = [expression._labeled(c) for c in columns]
         self.group = kwargs.pop('group', None)
         self.deferred = kwargs.pop('deferred', False)
+        self.no_instrument = kwargs.pop('_no_instrument', False)
         self.comparator_factory = kwargs.pop('comparator_factory', self.__class__.Comparator)
         self.descriptor = kwargs.pop('descriptor', None)
         self.extension = kwargs.pop('extension', None)
+        if kwargs:
+            raise TypeError(
+                "%s received unexpected keyword argument(s): %s" % (
+                    self.__class__.__name__, ', '.join(sorted(kwargs.keys()))))
+
         util.set_creation_order(self)
-        if self.deferred:
+        if self.no_instrument:
+            self.strategy_class = strategies.UninstrumentedColumnLoader
+        elif self.deferred:
             self.strategy_class = strategies.DeferredColumnLoader
         else:
             self.strategy_class = strategies.ColumnLoader
-
+    
+    def instrument_class(self, mapper):
+        if self.no_instrument:
+            return
+        
+        attributes.register_descriptor(
+            mapper.class_, 
+            self.key, 
+            comparator=self.comparator_factory(self, mapper), 
+            parententity=mapper,
+            property_=self
+            )
+        
     def do_init(self):
         super(ColumnProperty, self).do_init()
         if len(self.columns) > 1 and self.parent.primary_key.issuperset(self.columns):
@@ -57,8 +89,8 @@ class ColumnProperty(StrategizedProperty):
                 ("On mapper %s, primary key column '%s' is being combined "
                  "with distinct primary key column '%s' in attribute '%s'.  "
                  "Use explicit properties to give each column its own mapped "
-                 "attribute name.") % (str(self.parent), str(self.columns[1]),
-                                       str(self.columns[0]), self.key))
+                 "attribute name.") % (self.parent, self.columns[1],
+                                       self.columns[0], self.key))
 
     def copy(self):
         return ColumnProperty(deferred=self.deferred, group=self.group, *self.columns)
@@ -98,6 +130,7 @@ class ColumnProperty(StrategizedProperty):
             col = self.__clause_element__()
             return op(col._bind_param(other), col, **kwargs)
     
+    # TODO: legacy..do we need this ? (0.5)
     ColumnComparator = Comparator
     
     def __str__(self):
@@ -117,13 +150,14 @@ class CompositeProperty(ColumnProperty):
         self.composite_class = class_
         self.strategy_class = strategies.CompositeColumnLoader
 
-    def do_init(self):
-        super(ColumnProperty, self).do_init()
-        # TODO: similar PK check as ColumnProperty does ?
-
     def copy(self):
         return CompositeProperty(deferred=self.deferred, group=self.group, composite_class=self.composite_class, *self.columns)
 
+    def do_init(self):
+        # skip over ColumnProperty's do_init(),
+        # which issues assertions that do not apply to CompositeColumnProperty
+        super(ColumnProperty, self).do_init()
+
     def getattr(self, state, column):
         obj = state.get_impl(self.key).get(state)
         return self.get_col_value(column, obj)
@@ -176,6 +210,47 @@ class CompositeProperty(ColumnProperty):
     def __str__(self):
         return str(self.parent.class_.__name__) + "." + self.key
 
+class ConcreteInheritedProperty(MapperProperty):
+    extension = None
+
+    def setup(self, context, entity, path, adapter, **kwargs):
+        pass
+
+    def create_row_processor(self, selectcontext, path, mapper, row, adapter):
+        return (None, None)
+
+    def instrument_class(self, mapper):
+        def warn():
+            raise AttributeError("Concrete %s does not implement attribute %r at "
+                "the instance level.  Add this property explicitly to %s." % 
+                (self.parent, self.key, self.parent))
+
+        class NoninheritedConcreteProp(object):
+            def __set__(s, obj, value):
+                warn()
+            def __delete__(s, obj):
+                warn()
+            def __get__(s, obj, owner):
+                warn()
+
+        comparator_callable = None
+        # TODO: put this process into a deferred callable?
+        for m in self.parent.iterate_to_root():
+            p = m._get_property(self.key)
+            if not isinstance(p, ConcreteInheritedProperty):
+                comparator_callable = p.comparator_factory
+                break
+
+        attributes.register_descriptor(
+            mapper.class_, 
+            self.key, 
+            comparator=comparator_callable(self, mapper), 
+            parententity=mapper,
+            property_=self,
+            proxy_property=NoninheritedConcreteProp()
+            )
+
+
 class SynonymProperty(MapperProperty):
 
     extension = None
@@ -193,10 +268,9 @@ class SynonymProperty(MapperProperty):
     def create_row_processor(self, selectcontext, path, mapper, row, adapter):
         return (None, None)
 
-    def do_init(self):
+    def instrument_class(self, mapper):
         class_ = self.parent.class_
 
-        self.logger.info("register managed attribute %s on class %s" % (self.key, class_.__name__))
         if self.descriptor is None:
             class SynonymProp(object):
                 def __set__(s, obj, value):
@@ -219,8 +293,14 @@ class SynonymProperty(MapperProperty):
                     return prop.comparator_factory(prop, mapper)
             return comparator
 
-        strategies.DefaultColumnLoader(self)._register_attribute(
-            None, None, False, comparator_callable, proxy_property=self.descriptor)
+        attributes.register_descriptor(
+            mapper.class_, 
+            self.key, 
+            comparator=comparator_callable(self, mapper), 
+            parententity=mapper,
+            property_=self,
+            proxy_property=self.descriptor
+            )
 
     def merge(self, session, source, dest, dont_load, _recursive):
         pass
@@ -237,10 +317,17 @@ class ComparableProperty(MapperProperty):
         self.comparator_factory = comparator_factory
         util.set_creation_order(self)
 
-    def do_init(self):
+    def instrument_class(self, mapper):
         """Set up a proxy to the unmanaged descriptor."""
 
-        strategies.DefaultColumnLoader(self)._register_attribute(None, None, False, self.comparator_factory, proxy_property=self.descriptor)
+        attributes.register_descriptor(
+            mapper.class_, 
+            self.key, 
+            comparator=self.comparator_factory(self, mapper), 
+            parententity=mapper,
+            property_=self,
+            proxy_property=self.descriptor
+            )
 
     def setup(self, context, entity, path, adapter, **kwargs):
         pass
@@ -258,15 +345,22 @@ class RelationProperty(StrategizedProperty):
     """
 
     def __init__(self, argument,
-                 secondary=None, primaryjoin=None, secondaryjoin=None,
-                 foreign_keys=None, uselist=None, order_by=False, backref=None,
-                 _is_backref=False, post_update=False, cascade=False,
-                 extension=None, viewonly=False, lazy=True,
-                 collection_class=None, passive_deletes=False,
-                 passive_updates=True, remote_side=None,
-                 enable_typechecks=True, join_depth=None,
-                 comparator_factory=None, strategy_class=None,
-                 _local_remote_pairs=None, query_class=None):
+        secondary=None, primaryjoin=None,
+        secondaryjoin=None, 
+        foreign_keys=None,
+        uselist=None,
+        order_by=False,
+        backref=None,
+        back_populates=None,
+        post_update=False,
+        cascade=False, extension=None,
+        viewonly=False, lazy=True,
+        collection_class=None, passive_deletes=False,
+        passive_updates=True, remote_side=None,
+        enable_typechecks=True, join_depth=None,
+        comparator_factory=None,
+        strategy_class=None, _local_remote_pairs=None, query_class=None):
+
         self.uselist = uselist
         self.argument = argument
         self.secondary = secondary
@@ -287,7 +381,6 @@ class RelationProperty(StrategizedProperty):
         self.join_depth = join_depth
         self.local_remote_pairs = _local_remote_pairs
         self.extension = extension
-        self.__join_cache = {}
         self.comparator_factory = comparator_factory or RelationProperty.Comparator
         self.comparator = self.comparator_factory(self, None)
         util.set_creation_order(self)
@@ -304,7 +397,7 @@ class RelationProperty(StrategizedProperty):
         else:
             self.strategy_class = strategies.LazyLoader
 
-        self._reverse_property = None
+        self._reverse_property = set()
 
         if cascade is not False:
             self.cascade = CascadeOptions(cascade)
@@ -316,21 +409,37 @@ class RelationProperty(StrategizedProperty):
 
         self.order_by = order_by
 
-        if isinstance(backref, str):
+        self.back_populates = back_populates
+
+        if self.back_populates:
+            if backref:
+                raise sa_exc.ArgumentError("backref and back_populates keyword arguments are mutually exclusive")
+            self.backref = None
+        elif isinstance(backref, str):
             # propagate explicitly sent primary/secondary join conditions to the BackRef object if
             # just a string was sent
             if secondary is not None:
                 # reverse primary/secondary in case of a many-to-many
-                self.backref = BackRef(backref, primaryjoin=secondaryjoin, secondaryjoin=primaryjoin, passive_updates=self.passive_updates)
+                self.backref = BackRef(backref, primaryjoin=secondaryjoin, 
+                                    secondaryjoin=primaryjoin, passive_updates=self.passive_updates)
             else:
-                self.backref = BackRef(backref, primaryjoin=primaryjoin, secondaryjoin=secondaryjoin, passive_updates=self.passive_updates)
+                self.backref = BackRef(backref, primaryjoin=primaryjoin, 
+                                    secondaryjoin=secondaryjoin, passive_updates=self.passive_updates)
         else:
             self.backref = backref
-        self._is_backref = _is_backref
+
+    def instrument_class(self, mapper):
+        attributes.register_descriptor(
+            mapper.class_, 
+            self.key, 
+            comparator=self.comparator_factory(self, mapper), 
+            parententity=mapper,
+            property_=self
+            )
 
     class Comparator(PropComparator):
         def __init__(self, prop, mapper, of_type=None, adapter=None):
-            self.prop = self.property = prop
+            self.prop = prop
             self.mapper = mapper
             self.adapter = adapter
             if of_type:
@@ -341,14 +450,14 @@ class RelationProperty(StrategizedProperty):
             on the local side of generated expressions.
 
             """
-            return self.__class__(self.prop, self.mapper, getattr(self, '_of_type', None), adapter)
-            
+            return self.__class__(self.property, self.mapper, getattr(self, '_of_type', None), adapter)
+        
         @property
         def parententity(self):
-            return self.prop.parent
+            return self.property.parent
 
         def __clause_element__(self):
-            elem = self.prop.parent._with_polymorphic_selectable
+            elem = self.property.parent._with_polymorphic_selectable
             if self.adapter:
                 return self.adapter(elem)
             else:
@@ -361,7 +470,7 @@ class RelationProperty(StrategizedProperty):
             return op(self, *other, **kwargs)
 
         def of_type(self, cls):
-            return RelationProperty.Comparator(self.prop, self.mapper, cls)
+            return RelationProperty.Comparator(self.property, self.mapper, cls)
 
         def in_(self, other):
             raise NotImplementedError("in_() not yet supported for relations.  For a "
@@ -371,20 +480,20 @@ class RelationProperty(StrategizedProperty):
         
         def __eq__(self, other):
             if other is None:
-                if self.prop.direction in [ONETOMANY, MANYTOMANY]:
+                if self.property.direction in [ONETOMANY, MANYTOMANY]:
                     return ~self._criterion_exists()
                 else:
-                    return self.prop._optimized_compare(None, adapt_source=self.adapter)
-            elif self.prop.uselist:
+                    return self.property._optimized_compare(None, adapt_source=self.adapter)
+            elif self.property.uselist:
                 raise sa_exc.InvalidRequestError("Can't compare a collection to an object or collection; use contains() to test for membership.")
             else:
-                return self.prop._optimized_compare(other, adapt_source=self.adapter)
+                return self.property._optimized_compare(other, adapt_source=self.adapter)
 
         def _criterion_exists(self, criterion=None, **kwargs):
             if getattr(self, '_of_type', None):
                 target_mapper = self._of_type
                 to_selectable = target_mapper._with_polymorphic_selectable
-                if self.prop._is_self_referential():
+                if self.property._is_self_referential():
                     to_selectable = to_selectable.alias()
 
                 single_crit = target_mapper._single_table_criterion
@@ -402,10 +511,10 @@ class RelationProperty(StrategizedProperty):
                 source_selectable = None
                 
             pj, sj, source, dest, secondary, target_adapter = \
-                self.prop._create_joins(dest_polymorphic=True, dest_selectable=to_selectable, source_selectable=source_selectable)
+                self.property._create_joins(dest_polymorphic=True, dest_selectable=to_selectable, source_selectable=source_selectable)
 
             for k in kwargs:
-                crit = self.prop.mapper.class_manager[k] == kwargs[k]
+                crit = self.property.mapper.class_manager[k] == kwargs[k]
                 if criterion is None:
                     criterion = crit
                 else:
@@ -417,7 +526,7 @@ class RelationProperty(StrategizedProperty):
             if sj:
                 j = _orm_annotate(pj) & sj
             else:
-                j = _orm_annotate(pj, exclude=self.prop.remote_side)
+                j = _orm_annotate(pj, exclude=self.property.remote_side)
             
             if criterion and target_adapter:
                 # limit this adapter to annotated only?
@@ -434,34 +543,34 @@ class RelationProperty(StrategizedProperty):
             return sql.exists([1], crit, from_obj=dest).correlate(source)
 
         def any(self, criterion=None, **kwargs):
-            if not self.prop.uselist:
+            if not self.property.uselist:
                 raise sa_exc.InvalidRequestError("'any()' not implemented for scalar attributes. Use has().")
 
             return self._criterion_exists(criterion, **kwargs)
 
         def has(self, criterion=None, **kwargs):
-            if self.prop.uselist:
+            if self.property.uselist:
                 raise sa_exc.InvalidRequestError("'has()' not implemented for collections.  Use any().")
             return self._criterion_exists(criterion, **kwargs)
 
         def contains(self, other, **kwargs):
-            if not self.prop.uselist:
+            if not self.property.uselist:
                 raise sa_exc.InvalidRequestError("'contains' not implemented for scalar attributes.  Use ==")
-            clause = self.prop._optimized_compare(other, adapt_source=self.adapter)
+            clause = self.property._optimized_compare(other, adapt_source=self.adapter)
 
-            if self.prop.secondaryjoin:
+            if self.property.secondaryjoin:
                 clause.negation_clause = self.__negated_contains_or_equals(other)
 
             return clause
 
         def __negated_contains_or_equals(self, other):
-            if self.prop.direction == MANYTOONE:
+            if self.property.direction == MANYTOONE:
                 state = attributes.instance_state(other)
-                strategy = self.prop._get_strategy(strategies.LazyLoader)
+                strategy = self.property._get_strategy(strategies.LazyLoader)
                 
                 def state_bindparam(state, col):
                     o = state.obj() # strong ref
-                    return lambda: self.prop.mapper._get_committed_attr_by_column(o, col)
+                    return lambda: self.property.mapper._get_committed_attr_by_column(o, col)
                 
                 def adapt(col):
                     if self.adapter:
@@ -474,22 +583,27 @@ class RelationProperty(StrategizedProperty):
                         sql.or_(
                         adapt(x) != state_bindparam(state, y),
                         adapt(x) == None)
-                        for (x, y) in self.prop.local_remote_pairs])
+                        for (x, y) in self.property.local_remote_pairs])
                     
-            criterion = sql.and_(*[x==y for (x, y) in zip(self.prop.mapper.primary_key, self.prop.mapper.primary_key_from_instance(other))])
+            criterion = sql.and_(*[x==y for (x, y) in zip(self.property.mapper.primary_key, self.property.mapper.primary_key_from_instance(other))])
             return ~self._criterion_exists(criterion)
 
         def __ne__(self, other):
             if other is None:
-                if self.prop.direction == MANYTOONE:
-                    return sql.or_(*[x!=None for x in self.prop._foreign_keys])
+                if self.property.direction == MANYTOONE:
+                    return sql.or_(*[x!=None for x in self.property._foreign_keys])
                 else:
                     return self._criterion_exists()
-            elif self.prop.uselist:
+            elif self.property.uselist:
                 raise sa_exc.InvalidRequestError("Can't compare a collection to an object or collection; use contains() to test for membership.")
             else:
                 return self.__negated_contains_or_equals(other)
 
+        @util.memoized_property
+        def property(self):
+            self.prop.parent.compile()
+            return self.prop
+
     def compare(self, op, value, value_is_parent=False):
         if op == operators.eq:
             if value is None:
@@ -512,8 +626,11 @@ class RelationProperty(StrategizedProperty):
         return str(self.parent.class_.__name__) + "." + self.key
 
     def merge(self, session, source, dest, dont_load, _recursive):
-        if not dont_load and self._reverse_property and (source, self._reverse_property) in _recursive:
-            return
+        if not dont_load:
+            # TODO: no test coverage for recursive check
+            for r in self._reverse_property:
+                if (source, r) in _recursive:
+                    return
 
         source_state = attributes.instance_state(source)
         dest_state = attributes.instance_state(dest)
@@ -547,7 +664,7 @@ class RelationProperty(StrategizedProperty):
                 obj = session.merge(current, dont_load=dont_load, _recursive=_recursive)
                 if obj is not None:
                     if dont_load:
-                        dest.__dict__[self.key] = obj
+                        dest_state.dict[self.key] = obj
                     else:
                         setattr(dest, self.key, obj)
 
@@ -567,42 +684,47 @@ class RelationProperty(StrategizedProperty):
             for c in instances:
                 if c is not None and c not in visited_instances and (halt_on is None or not halt_on(c)):
                     if not isinstance(c, self.mapper.class_):
-                        raise AssertionError("Attribute '%s' on class '%s' doesn't handle objects of type '%s'" % (self.key, str(self.parent.class_), str(c.__class__)))
+                        raise AssertionError("Attribute '%s' on class '%s' doesn't handle objects "
+                                    "of type '%s'" % (self.key, str(self.parent.class_), str(c.__class__)))
                     visited_instances.add(c)
 
                     # cascade using the mapper local to this object, so that its individual properties are located
                     instance_mapper = object_mapper(c)
                     yield (c, instance_mapper, attributes.instance_state(c))
 
-    def _get_target_class(self):
-        """Return the target class of the relation, even if the
-        property has not been initialized yet.
-
-        """
-        if isinstance(self.argument, type):
-            return self.argument
-        else:
-            return self.argument.class_
-
+    def _add_reverse_property(self, key):
+        other = self.mapper._get_property(key)
+        self._reverse_property.add(other)
+        other._reverse_property.add(self)
+        
+        if not other._get_target().common_parent(self.parent):
+            raise sa_exc.ArgumentError("reverse_property %r on relation %s references "
+                    "relation %s, which does not reference mapper %s" % (key, self, other, self.parent))
+        
     def do_init(self):
-        self._determine_targets()
+        self._get_target()
+        self._process_dependent_arguments()
         self._determine_joins()
         self._determine_synchronize_pairs()
         self._determine_direction()
         self._determine_local_remote_pairs()
         self._post_init()
 
-    def _determine_targets(self):
-        if isinstance(self.argument, type):
-            self.mapper = mapper.class_mapper(self.argument, compile=False)
-        elif isinstance(self.argument, mapper.Mapper):
-            self.mapper = self.argument
-        elif util.callable(self.argument):
-            # accept a callable to suit various deferred-configurational schemes
-            self.mapper = mapper.class_mapper(self.argument(), compile=False)
-        else:
-            raise sa_exc.ArgumentError("relation '%s' expects a class or a mapper argument (received: %s)" % (self.key, type(self.argument)))
-        assert isinstance(self.mapper, mapper.Mapper), self.mapper
+    def _get_target(self):
+        if not hasattr(self, 'mapper'):
+            if isinstance(self.argument, type):
+                self.mapper = mapper.class_mapper(self.argument, compile=False)
+            elif isinstance(self.argument, mapper.Mapper):
+                self.mapper = self.argument
+            elif util.callable(self.argument):
+                # accept a callable to suit various deferred-configurational schemes
+                self.mapper = mapper.class_mapper(self.argument(), compile=False)
+            else:
+                raise sa_exc.ArgumentError("relation '%s' expects a class or a mapper argument (received: %s)" % (self.key, type(self.argument)))
+            assert isinstance(self.mapper, mapper.Mapper), self.mapper
+        return self.mapper
+        
+    def _process_dependent_arguments(self):
 
         # accept callables for other attributes which may require deferred initialization
         for attr in ('order_by', 'primaryjoin', 'secondaryjoin', 'secondary', '_foreign_keys', 'remote_side'):
@@ -787,7 +909,11 @@ class RelationProperty(StrategizedProperty):
                     "- foreign key columns are present in both the parent and "
                     "the child's mapped tables.  Specify 'foreign_keys' "
                     "argument." % (str(self)))
-
+        
+        if self.cascade.delete_orphan and self.direction is MANYTOMANY:
+            util.warn("On %s, delete-orphan cascade is not supported on a "
+                    "many-to-many relation.  This will raise an error in 0.6." % self)
+        
     def _determine_local_remote_pairs(self):
         if not self.local_remote_pairs:
             if self.remote_side:
@@ -855,6 +981,11 @@ class RelationProperty(StrategizedProperty):
 
         # primary property handler, set up class attributes
         if self.is_primary():
+            if self.back_populates:
+                self.extension = util.to_list(self.extension) or []
+                self.extension.append(attributes.GenericBackrefExtension(self.back_populates))
+                self._add_reverse_property(self.back_populates)
+            
             if self.backref is not None:
                 self.backref.compile(self)
         elif not mapper.class_mapper(self.parent.class_, compile=False)._get_property(self.key, raiseerr=False):
@@ -862,7 +993,7 @@ class RelationProperty(StrategizedProperty):
                 "a non-primary mapper on class '%s'.  New relations can only be "
                 "added to the primary mapper, i.e. the very first "
                 "mapper created for class '%s' " % (self.key, self.parent.class_.__name__, self.parent.class_.__name__))
-
+        
         super(RelationProperty, self).do_init()
 
     def _refers_to_parent_table(self):
@@ -872,12 +1003,6 @@ class RelationProperty(StrategizedProperty):
         return self.mapper.common_parent(self.parent)
 
     def _create_joins(self, source_polymorphic=False, source_selectable=None, dest_polymorphic=False, dest_selectable=None, of_type=None):
-        key = util.WeakCompositeKey(source_polymorphic, source_selectable, dest_polymorphic, dest_selectable, of_type)
-        try:
-            return self.__join_cache[key]
-        except KeyError:
-            pass
-
         if source_selectable is None:
             if source_polymorphic and self.parent.with_polymorphic:
                 source_selectable = self.parent._with_polymorphic_selectable
@@ -944,10 +1069,9 @@ class RelationProperty(StrategizedProperty):
         else:
             target_adapter = None
 
-        self.__join_cache[key] = ret = (primaryjoin, secondaryjoin, 
+        return (primaryjoin, secondaryjoin, 
                 (source_selectable or self.parent.local_table), 
                 (dest_selectable or self.mapper.local_table), secondary, target_adapter)
-        return ret
 
     def _get_join(self, parent, primary=True, secondary=True, polymorphic_parent=True):
         """deprecated.  use primary_join_against(), secondary_join_against(), full_join_against()"""
@@ -973,7 +1097,12 @@ log.class_logger(RelationProperty)
 class BackRef(object):
     """Attached to a RelationProperty to indicate a complementary reverse relationship.
 
-    Can optionally create the complementing RelationProperty if one does not exist already."""
+    Handles the job of creating the opposite RelationProperty according to configuration.
+    
+    Alternatively, two explicit RelationProperty objects can be associated bidirectionally
+    using the back_populates keyword argument on each.
+    
+    """
 
     def __init__(self, key, _prop=None, **kwargs):
         self.key = key
@@ -1006,13 +1135,11 @@ class BackRef(object):
 
             relation = RelationProperty(parent, prop.secondary, pj, sj,
                                       backref=BackRef(prop.key, _prop=prop),
-                                      _is_backref=True,
                                       **self.kwargs)
 
-            mapper._compile_property(self.key, relation);
+            mapper._configure_property(self.key, relation);
 
-            prop._reverse_property = mapper._get_property(self.key)
-            mapper._get_property(self.key)._reverse_property = prop
+            prop._add_reverse_property(self.key)
 
         else:
             raise sa_exc.ArgumentError("Error creating backref '%s' on relation '%s': "
@@ -1021,3 +1148,5 @@ class BackRef(object):
 mapper.ColumnProperty = ColumnProperty
 mapper.SynonymProperty = SynonymProperty
 mapper.ComparableProperty = ComparableProperty
+mapper.RelationProperty = RelationProperty
+mapper.ConcreteInheritedProperty = ConcreteInheritedProperty
