@@ -1,11 +1,11 @@
 import testenv; testenv.configure_for_tests()
 import sys, time, threading
-
-from sqlalchemy import *
-from sqlalchemy.orm import *
-from testlib import *
+from testlib.sa import create_engine, MetaData, Table, Column, INT, VARCHAR, \
+     Sequence, select, Integer, String, func, text
+from testlib import TestBase, testing
 
 
+users, metadata = None, None
 class TransactionTest(TestBase):
     def setUpAll(self):
         global users, metadata
@@ -22,7 +22,7 @@ class TransactionTest(TestBase):
     def tearDownAll(self):
         users.drop(testing.db)
 
-    def testcommits(self):
+    def test_commits(self):
         connection = testing.db.connect()
         transaction = connection.begin()
         connection.execute(users.insert(), user_id=1, user_name='user1')
@@ -38,7 +38,7 @@ class TransactionTest(TestBase):
         assert len(result.fetchall()) == 3
         transaction.commit()
 
-    def testrollback(self):
+    def test_rollback(self):
         """test a basic rollback"""
         connection = testing.db.connect()
         transaction = connection.begin()
@@ -51,7 +51,7 @@ class TransactionTest(TestBase):
         assert len(result.fetchall()) == 0
         connection.close()
 
-    def testraise(self):
+    def test_raise(self):
         connection = testing.db.connect()
 
         transaction = connection.begin()
@@ -69,8 +69,7 @@ class TransactionTest(TestBase):
         assert len(result.fetchall()) == 0
         connection.close()
 
-    @testing.exclude('mysql', '<', (5, 0, 3))
-    def testnestedrollback(self):
+    def test_nested_rollback(self):
         connection = testing.db.connect()
 
         try:
@@ -99,8 +98,7 @@ class TransactionTest(TestBase):
                 connection.close()
 
 
-    @testing.exclude('mysql', '<', (5, 0, 3))
-    def testnesting(self):
+    def test_nesting(self):
         connection = testing.db.connect()
         transaction = connection.begin()
         connection.execute(users.insert(), user_id=1, user_name='user1')
@@ -117,8 +115,7 @@ class TransactionTest(TestBase):
         assert len(result.fetchall()) == 0
         connection.close()
 
-    @testing.exclude('mysql', '<', (5, 0, 3))
-    def testclose(self):
+    def test_close(self):
         connection = testing.db.connect()
         transaction = connection.begin()
         connection.execute(users.insert(), user_id=1, user_name='user1')
@@ -138,8 +135,7 @@ class TransactionTest(TestBase):
         assert len(result.fetchall()) == 5
         connection.close()
 
-    @testing.exclude('mysql', '<', (5, 0, 3))
-    def testclose2(self):
+    def test_close2(self):
         connection = testing.db.connect()
         transaction = connection.begin()
         connection.execute(users.insert(), user_id=1, user_name='user1')
@@ -159,10 +155,8 @@ class TransactionTest(TestBase):
         assert len(result.fetchall()) == 0
         connection.close()
 
-
-    @testing.unsupported('sqlite', 'mssql', 'firebird', 'sybase', 'access')
-    @testing.exclude('mysql', '<', (5, 0, 3))
-    def testnestedsubtransactionrollback(self):
+    @testing.requires.savepoints
+    def test_nested_subtransaction_rollback(self):
         connection = testing.db.connect()
         transaction = connection.begin()
         connection.execute(users.insert(), user_id=1, user_name='user1')
@@ -178,9 +172,8 @@ class TransactionTest(TestBase):
         )
         connection.close()
 
-    @testing.unsupported('sqlite', 'mssql', 'firebird', 'sybase', 'access')
-    @testing.exclude('mysql', '<', (5, 0, 3))
-    def testnestedsubtransactioncommit(self):
+    @testing.requires.savepoints
+    def test_nested_subtransaction_commit(self):
         connection = testing.db.connect()
         transaction = connection.begin()
         connection.execute(users.insert(), user_id=1, user_name='user1')
@@ -196,9 +189,8 @@ class TransactionTest(TestBase):
         )
         connection.close()
 
-    @testing.unsupported('sqlite', 'mssql', 'firebird', 'sybase', 'access')
-    @testing.exclude('mysql', '<', (5, 0, 3))
-    def testrollbacktosubtransaction(self):
+    @testing.requires.savepoints
+    def test_rollback_to_subtransaction(self):
         connection = testing.db.connect()
         transaction = connection.begin()
         connection.execute(users.insert(), user_id=1, user_name='user1')
@@ -216,10 +208,8 @@ class TransactionTest(TestBase):
         )
         connection.close()
 
-    @testing.unsupported('sqlite', 'mssql', 'firebird', 'sybase', 'access',
-                         'oracle', 'maxdb')
-    @testing.exclude('mysql', '<', (5, 0, 3))
-    def testtwophasetransaction(self):
+    @testing.requires.two_phase_transactions
+    def test_two_phase_transaction(self):
         connection = testing.db.connect()
 
         transaction = connection.begin_twophase()
@@ -246,10 +236,9 @@ class TransactionTest(TestBase):
         )
         connection.close()
 
-    @testing.unsupported('sqlite', 'mssql', 'firebird', 'sybase', 'access',
-                         'oracle', 'maxdb')
-    @testing.exclude('mysql', '<', (5, 0, 3))
-    def testmixedtwophasetransaction(self):
+    @testing.requires.two_phase_transactions
+    @testing.requires.savepoints
+    def test_mixed_two_phase_transaction(self):
         connection = testing.db.connect()
 
         transaction = connection.begin_twophase()
@@ -281,11 +270,9 @@ class TransactionTest(TestBase):
         )
         connection.close()
 
-    @testing.unsupported('sqlite', 'mssql', 'firebird', 'sybase', 'access',
-                         'oracle', 'maxdb')
-    # fixme: see if this is still true and/or can be convert to fails_on()
-    @testing.unsupported('mysql')
-    def testtwophaserecover(self):
+    @testing.requires.two_phase_transactions
+    @testing.fails_on('mysql', 'FIXME: unknown')
+    def test_two_phase_recover(self):
         # MySQL recovery doesn't currently seem to work correctly
         # Prepared transactions disappear when connections are closed and even
         # when they aren't it doesn't seem possible to use the recovery id.
@@ -316,10 +303,8 @@ class TransactionTest(TestBase):
         )
         connection2.close()
 
-    @testing.unsupported('sqlite', 'mssql', 'firebird', 'sybase', 'access',
-                         'oracle', 'maxdb')
-    @testing.exclude('mysql', '<', (5, 0, 3))
-    def testmultipletwophase(self):
+    @testing.requires.two_phase_transactions
+    def test_multiple_two_phase(self):
         conn = testing.db.connect()
 
         xa = conn.begin_twophase()
@@ -354,8 +339,7 @@ class AutoRollbackTest(TestBase):
     def tearDownAll(self):
         metadata.drop_all(testing.db)
 
-    @testing.unsupported('sqlite')
-    def testrollback_deadlock(self):
+    def test_rollback_deadlock(self):
         """test that returning connections to the pool clears any object locks."""
         conn1 = testing.db.connect()
         conn2 = testing.db.connect()
@@ -375,12 +359,13 @@ class AutoRollbackTest(TestBase):
         users.drop(conn2)
         conn2.close()
 
+foo = None
 class ExplicitAutoCommitTest(TestBase):
-    """test the 'autocommit' flag on select() and text() objects.  
-    
+    """test the 'autocommit' flag on select() and text() objects.
+
     Requires Postgres so that we may define a custom function which modifies the database.
     """
-    
+
     __only_on__ = 'postgres'
 
     def setUpAll(self):
@@ -392,13 +377,13 @@ class ExplicitAutoCommitTest(TestBase):
 
     def tearDown(self):
         foo.delete().execute()
-        
+
     def tearDownAll(self):
         testing.db.execute("drop function insert_foo(varchar)")
         metadata.drop_all()
-    
+
     def test_control(self):
-        # test that not using autocommit does not commit 
+        # test that not using autocommit does not commit
         conn1 = testing.db.connect()
         conn2 = testing.db.connect()
 
@@ -412,44 +397,45 @@ class ExplicitAutoCommitTest(TestBase):
         trans.commit()
 
         assert conn2.execute(select([foo.c.data])).fetchall() == [('data1',), ('moredata',)]
-        
+
         conn1.close()
         conn2.close()
-        
+
     def test_explicit_compiled(self):
         conn1 = testing.db.connect()
         conn2 = testing.db.connect()
-        
+
         conn1.execute(select([func.insert_foo('data1')], autocommit=True))
         assert conn2.execute(select([foo.c.data])).fetchall() == [('data1',)]
 
         conn1.execute(select([func.insert_foo('data2')]).autocommit())
         assert conn2.execute(select([foo.c.data])).fetchall() == [('data1',), ('data2',)]
-        
+
         conn1.close()
         conn2.close()
-    
+
     def test_explicit_text(self):
         conn1 = testing.db.connect()
         conn2 = testing.db.connect()
-        
+
         conn1.execute(text("select insert_foo('moredata')", autocommit=True))
         assert conn2.execute(select([foo.c.data])).fetchall() == [('moredata',)]
-        
+
         conn1.close()
         conn2.close()
 
     def test_implicit_text(self):
         conn1 = testing.db.connect()
         conn2 = testing.db.connect()
-        
+
         conn1.execute(text("insert into foo (data) values ('implicitdata')"))
         assert conn2.execute(select([foo.c.data])).fetchall() == [('implicitdata',)]
-        
+
         conn1.close()
         conn2.close()
-        
-    
+
+
+tlengine = None
 class TLTransactionTest(TestBase):
     def setUpAll(self):
         global users, metadata, tlengine
@@ -467,6 +453,10 @@ class TLTransactionTest(TestBase):
         users.drop(tlengine)
         tlengine.dispose()
 
+    def test_nested_unsupported(self):
+        self.assertRaises(NotImplementedError, tlengine.contextual_connect().begin_nested)
+        self.assertRaises(NotImplementedError, tlengine.begin_nested)
+        
     def test_connection_close(self):
         """test that when connections are closed for real, transactions are rolled back and disposed."""
 
@@ -502,7 +492,7 @@ class TLTransactionTest(TestBase):
         finally:
             external_connection.close()
 
-    def testrollback(self):
+    def test_rollback(self):
         """test a basic rollback"""
         tlengine.begin()
         tlengine.execute(users.insert(), user_id=1, user_name='user1')
@@ -517,7 +507,7 @@ class TLTransactionTest(TestBase):
         finally:
             external_connection.close()
 
-    def testcommit(self):
+    def test_commit(self):
         """test a basic commit"""
         tlengine.begin()
         tlengine.execute(users.insert(), user_id=1, user_name='user1')
@@ -532,7 +522,7 @@ class TLTransactionTest(TestBase):
         finally:
             external_connection.close()
 
-    def testcommits(self):
+    def test_commits(self):
         assert tlengine.connect().execute("select count(1) from query_users").scalar() == 0
 
         connection = tlengine.contextual_connect()
@@ -551,7 +541,7 @@ class TLTransactionTest(TestBase):
         assert len(l) == 3, "expected 3 got %d" % len(l)
         transaction.commit()
 
-    def testrollback_off_conn(self):
+    def test_rollback_off_conn(self):
         # test that a TLTransaction opened off a TLConnection allows that
         # TLConnection to be aware of the transactional context
         conn = tlengine.contextual_connect()
@@ -568,7 +558,7 @@ class TLTransactionTest(TestBase):
         finally:
             external_connection.close()
 
-    def testmorerollback_off_conn(self):
+    def test_morerollback_off_conn(self):
         # test that an existing TLConnection automatically takes place in a TLTransaction
         # opened on a second TLConnection
         conn = tlengine.contextual_connect()
@@ -586,7 +576,7 @@ class TLTransactionTest(TestBase):
         finally:
             external_connection.close()
 
-    def testcommit_off_conn(self):
+    def test_commit_off_connection(self):
         conn = tlengine.contextual_connect()
         trans = conn.begin()
         conn.execute(users.insert(), user_id=1, user_name='user1')
@@ -601,9 +591,7 @@ class TLTransactionTest(TestBase):
         finally:
             external_connection.close()
 
-    @testing.unsupported('sqlite')
-    @testing.exclude('mysql', '<', (5, 0, 3))
-    def testnesting(self):
+    def test_nesting(self):
         """tests nesting of transactions"""
         external_connection = tlengine.connect()
         self.assert_(external_connection.connection is not tlengine.contextual_connect().connection)
@@ -621,8 +609,7 @@ class TLTransactionTest(TestBase):
         finally:
             external_connection.close()
 
-    @testing.exclude('mysql', '<', (5, 0, 3))
-    def testmixednesting(self):
+    def test_mixed_nesting(self):
         """tests nesting of transactions off the TLEngine directly inside of
         tranasctions off the connection from the TLEngine"""
         external_connection = tlengine.connect()
@@ -650,8 +637,7 @@ class TLTransactionTest(TestBase):
         finally:
             external_connection.close()
 
-    @testing.exclude('mysql', '<', (5, 0, 3))
-    def testmoremixednesting(self):
+    def test_more_mixed_nesting(self):
         """tests nesting of transactions off the connection from the TLEngine
         inside of tranasctions off thbe TLEngine directly."""
         external_connection = tlengine.connect()
@@ -674,24 +660,7 @@ class TLTransactionTest(TestBase):
         finally:
             external_connection.close()
 
-    @testing.exclude('mysql', '<', (5, 0, 3))
-    def testsessionnesting(self):
-        class User(object):
-            pass
-        try:
-            mapper(User, users)
-
-            sess = create_session(bind=tlengine)
-            tlengine.begin()
-            u = User()
-            sess.save(u)
-            sess.flush()
-            tlengine.commit()
-        finally:
-            clear_mappers()
-
-
-    def testconnections(self):
+    def test_connections(self):
         """tests that contextual_connect is threadlocal"""
         c1 = tlengine.contextual_connect()
         c2 = tlengine.contextual_connect()
@@ -699,7 +668,32 @@ class TLTransactionTest(TestBase):
         c2.close()
         assert c1.connection.connection is not None
 
+    @testing.requires.two_phase_transactions
+    def test_two_phase_transaction(self):
+        tlengine.begin_twophase()
+        tlengine.execute(users.insert(), user_id=1, user_name='user1')
+        tlengine.prepare()
+        tlengine.commit()
 
+        tlengine.begin_twophase()
+        tlengine.execute(users.insert(), user_id=2, user_name='user2')
+        tlengine.commit()
+
+        tlengine.begin_twophase()
+        tlengine.execute(users.insert(), user_id=3, user_name='user3')
+        tlengine.rollback()
+
+        tlengine.begin_twophase()
+        tlengine.execute(users.insert(), user_id=4, user_name='user4')
+        tlengine.prepare()
+        tlengine.rollback()
+
+        self.assertEquals(
+            tlengine.execute(select([users.c.user_id]).order_by(users.c.user_id)).fetchall(),
+            [(1,),(2,)]
+        )
+
+counters = None
 class ForUpdateTest(TestBase):
     def setUpAll(self):
         global counters, metadata
@@ -742,15 +736,18 @@ class ForUpdateTest(TestBase):
                 break
         con.close()
 
-    @testing.unsupported('sqlite', 'mssql', 'firebird', 'sybase', 'access')
-
-    def testqueued_update(self):
+    @testing.crashes('mssql', 'FIXME: unknown')
+    @testing.crashes('firebird', 'FIXME: unknown')
+    @testing.crashes('sybase', 'FIXME: unknown')
+    @testing.crashes('access', 'FIXME: unknown')
+    @testing.requires.independent_connections
+    def test_queued_update(self):
         """Test SELECT FOR UPDATE with concurrent modifications.
 
         Runs concurrent modifications on a single row in the users table,
         with each mutator trying to increment a value stored in user_name.
-        """
 
+        """
         db = testing.db
         db.execute(counters.insert(), counter_id=1, counter_value=0)
 
@@ -767,7 +764,7 @@ class ForUpdateTest(TestBase):
             thread.join()
 
         for e in errors:
-            sys.stderr.write("Failure: %s\n" % e)
+            sys.stdout.write("Failure: %s\n" % e)
 
         self.assert_(len(errors) == 0)
 
@@ -805,8 +802,12 @@ class ForUpdateTest(TestBase):
 
         return errors
 
-    @testing.unsupported('sqlite', 'mssql', 'firebird', 'sybase', 'access')
-    def testqueued_select(self):
+    @testing.crashes('mssql', 'FIXME: unknown')
+    @testing.crashes('firebird', 'FIXME: unknown')
+    @testing.crashes('sybase', 'FIXME: unknown')
+    @testing.crashes('access', 'FIXME: unknown')
+    @testing.requires.independent_connections
+    def test_queued_select(self):
         """Simple SELECT FOR UPDATE conflict test"""
 
         errors = self._threaded_overlap(2, [(1,2,3),(3,4,5)])
@@ -814,9 +815,13 @@ class ForUpdateTest(TestBase):
             sys.stderr.write("Failure: %s\n" % e)
         self.assert_(len(errors) == 0)
 
-    @testing.unsupported('sqlite', 'mysql', 'mssql', 'firebird',
-                         'sybase', 'access')
-    def testnowait_select(self):
+    @testing.crashes('mssql', 'FIXME: unknown')
+    @testing.fails_on('mysql', 'No support for NOWAIT')
+    @testing.crashes('firebird', 'FIXME: unknown')
+    @testing.crashes('sybase', 'FIXME: unknown')
+    @testing.crashes('access', 'FIXME: unknown')
+    @testing.requires.independent_connections
+    def test_nowait_select(self):
         """Simple SELECT FOR UPDATE NOWAIT conflict test"""
 
         errors = self._threaded_overlap(2, [(1,2,3),(3,4,5)],
