@@ -31,7 +31,7 @@ class O2MCascadeTest(_fixtures.FixtureTest):
                  Order(description='someotherorder')])
         sess.add(u)
         sess.flush()
-        sess.clear()
+        sess.expunge_all()
 
         u = sess.query(User).get(u.id)
         eq_(u, User(name='jack',
@@ -40,7 +40,7 @@ class O2MCascadeTest(_fixtures.FixtureTest):
 
         u.orders=[Order(description="order 3"), Order(description="order 4")]
         sess.flush()
-        sess.clear()
+        sess.expunge_all()
 
         u = sess.query(User).get(u.id)
         eq_(u, User(name='jack',
@@ -82,7 +82,7 @@ class O2MCascadeTest(_fixtures.FixtureTest):
                             Address(email_address="address2")])
         sess.add(u)
         sess.flush()
-        sess.clear()
+        sess.expunge_all()
         assert addresses.count().scalar() == 2
         assert users.count().scalar() == 1
 
@@ -116,7 +116,7 @@ class O2MCascadeTest(_fixtures.FixtureTest):
         u2 = User(name='newuser', orders=[o])
         sess.add(u2)
         sess.flush()
-        sess.clear()
+        sess.expunge_all()
         assert users.count().scalar() == 1
         assert orders.count().scalar() == 1
         eq_(sess.query(User).all(),
@@ -231,7 +231,7 @@ class NoSaveCascadeTest(_fixtures.FixtureTest):
         assert u1 not in sess
         assert o1 in sess
         
-        sess.clear()
+        sess.expunge_all()
         
         o1 = Order()
         u1 = User(orders=[o1])
@@ -255,7 +255,7 @@ class NoSaveCascadeTest(_fixtures.FixtureTest):
         assert o1 not in sess
         assert u1 in sess
         
-        sess.clear()
+        sess.expunge_all()
 
         u1 = User()
         o1 = Order()
@@ -280,7 +280,7 @@ class NoSaveCascadeTest(_fixtures.FixtureTest):
         assert i1 in sess
         assert k1 not in sess
         
-        sess.clear()
+        sess.expunge_all()
         
         i1 = Item()
         k1 = Keyword()
@@ -383,12 +383,12 @@ class M2OCascadeTest(_base.MappedTest):
         jack = sess.query(User).filter_by(name="jack").one()
         p = jack.pref
         e = jack.pref.extra[0]
-        sess.clear()
+        sess.expunge_all()
 
         jack.pref = None
-        sess.update(jack)
-        sess.update(p)
-        sess.update(e)
+        sess.add(jack)
+        sess.add(p)
+        sess.add(e)
         assert p in sess
         assert e in sess
         sess.flush()
@@ -626,20 +626,33 @@ class M2OCascadeDeleteOrphanTest(_base.MappedTest):
         eq_(sess.query(T3).all(), [])
 
 class M2MCascadeTest(_base.MappedTest):
+    """delete-orphan cascade is deprecated on many-to-many."""
+    
     def define_tables(self, metadata):
         Table('a', metadata,
             Column('id', Integer, primary_key=True),
-            Column('data', String(30)))
+            Column('data', String(30)),
+            test_needs_fk=True
+            )
         Table('b', metadata,
             Column('id', Integer, primary_key=True),
-            Column('data', String(30)))
+            Column('data', String(30)),
+            test_needs_fk=True
+            
+            )
         Table('atob', metadata,
             Column('aid', Integer, ForeignKey('a.id')),
-            Column('bid', Integer, ForeignKey('b.id')))
+            Column('bid', Integer, ForeignKey('b.id')),
+            test_needs_fk=True
+            
+            )
         Table('c', metadata,
               Column('id', Integer, primary_key=True),
               Column('data', String(30)),
-              Column('bid', Integer, ForeignKey('b.id')))
+              Column('bid', Integer, ForeignKey('b.id')),
+              test_needs_fk=True
+              
+              )
 
     def setup_classes(self):
         class A(_fixtures.Base):
@@ -649,6 +662,7 @@ class M2MCascadeTest(_base.MappedTest):
         class C(_fixtures.Base):
             pass
 
+    @testing.emits_warning(".*not supported on a many-to-many")
     @testing.resolve_artifact_names
     def test_delete_orphan(self):
         mapper(A, a, properties={
@@ -670,6 +684,7 @@ class M2MCascadeTest(_base.MappedTest):
         assert b.count().scalar() == 0
         assert a.count().scalar() == 1
 
+    @testing.emits_warning(".*not supported on a many-to-many")
     @testing.resolve_artifact_names
     def test_delete_orphan_cascades(self):
         mapper(A, a, properties={
@@ -693,6 +708,7 @@ class M2MCascadeTest(_base.MappedTest):
         assert a.count().scalar() == 1
         assert c.count().scalar() == 0
 
+    @testing.emits_warning(".*not supported on a many-to-many")
     @testing.resolve_artifact_names
     def test_cascade_delete(self):
         mapper(A, a, properties={
@@ -711,6 +727,39 @@ class M2MCascadeTest(_base.MappedTest):
         assert b.count().scalar() == 0
         assert a.count().scalar() == 0
 
+    @testing.emits_warning(".*not supported on a many-to-many")
+    @testing.fails_on_everything_except('sqlite')
+    @testing.resolve_artifact_names
+    def test_this_doesnt_work(self):
+        """illustrates why cascade with m2m should not be supported
+            (i.e. many parents...)
+            
+        """
+        mapper(A, a, properties={
+            'bs':relation(B, secondary=atob, cascade="all, delete-orphan")
+        })
+        mapper(B, b)
+
+        sess = create_session()
+        b1 =B(data='b1')
+        a1 = A(data='a1', bs=[b1])
+        a2 = A(data='a2', bs=[b1])
+        sess.add(a1)
+        sess.add(a2)
+        sess.flush()
+
+        sess.delete(a1)
+        
+        # this raises an integrity error on DBs that support FKs
+        sess.flush()
+        
+        # still a row present !
+        assert atob.count().scalar() ==1
+        
+        # but no bs !
+        assert b.count().scalar() == 0
+        assert a.count().scalar() == 1
+
 
 class UnsavedOrphansTest(_base.MappedTest):
     """Pending entities that are orphans"""
@@ -791,7 +840,7 @@ class UnsavedOrphansTest(_base.MappedTest):
         u.addresses.remove(a1)
         assert a1 in s
         s.flush()
-        s.clear()
+        s.expunge_all()
         eq_(s.query(Address).all(), [Address(email_address='ad1')])
 
 
@@ -959,7 +1008,7 @@ class DoubleParentOrphanTest(_base.MappedTest):
         b1 = Business(description='business1', address=Address(street='address2'))
         session.add_all((h1,b1))
         session.flush()
-        session.clear()
+        session.expunge_all()
 
         eq_(session.query(Home).get(h1.id), Home(description='home1', address=Address(street='address1')))
         eq_(session.query(Business).get(b1.id), Business(description='business1', address=Address(street='address2')))
@@ -1017,7 +1066,7 @@ class CollectionAssignmentOrphanTest(_base.MappedTest):
         sess.add(a1)
         sess.flush()
 
-        sess.clear()
+        sess.expunge_all()
 
         eq_(sess.query(A).get(a1.id),
             A(name='a1', bs=[B(name='b1'), B(name='b2'), B(name='b3')]))
@@ -1029,7 +1078,7 @@ class CollectionAssignmentOrphanTest(_base.MappedTest):
         a1.bs[1].foo='b3modified'
         sess.flush()
 
-        sess.clear()
+        sess.expunge_all()
         eq_(sess.query(A).get(a1.id),
             A(name='a1', bs=[B(name='b1'), B(name='b2'), B(name='b3')]))
 
